/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR5969 Lab1 Low Power: ADC12B, Sample A2, AVcc Ref,
//  Set P1.0 if A2 > 0.5*AVcc
//
//   Description: Low Power ADC usage.
//   Repeated samples are made on A2 with reference to AVcc. Successive sample
//   and conversions are triggered by TA0.1, and the window comparator
//   triggers interrupts to wake from LPM0. This allows the device to remain in
//   LPM0 through all conversions until the conversion result from A2 crosses
//   the threshold at 0.5/AVcc. If A2 > 0.5*AVcc, The LED on P1.0 is set and
//   the window comparator "Lo" interrupt is enabled to catch the next time the
//   input crosses the threshold. Else, the LED is reset and the window
//   comparator "Hi" interrupt is enabled.
//
//
//              MSP430FR5969
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//        >---|P1.2/A2      P1.0|-->LED
//
//   William Goh
//   Texas Instruments Inc.
//   October 2013
//   Built with Code Composer Studio v5.5 and MSP430Ware 1.60.01.11
//******************************************************************************
#include <msp430.h>
#include "driverlib.h"

void boardInit(void);

unsigned char data = 0;

void main(void)
{
    // Stop WDT
    WDT_A_hold(WDT_A_BASE);

    //initialize clocks and pins
    boardInit();

    //Set P1.0 as an output pin.
    GPIO_setAsOutputPin(GPIO_PORT_P1,
            GPIO_PIN0);

    //Set P1.0 as Output Low.
    GPIO_setOutputLowOnPin(GPIO_PORT_P1,
            GPIO_PIN0);

    //Set P1.2 as Ternary Module Function Output = A2 (see datasheet)
    GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P1,
            GPIO_PIN2,
            GPIO_TERNARY_MODULE_FUNCTION);

    //Initialize timer to trigger ADC conversions
    //Configure timer in up mode
    TIMER_A_configureUpMode(TIMER_A0_BASE,          //TA0
            TIMER_A_CLOCKSOURCE_ACLK,               //ACLK (VLO = 9kHz)
            TIMER_A_CLOCKSOURCE_DIVIDER_1,          //ACLK/1
            90,                                     //9kHz/90 = 100Hz freq.
            TIMER_A_TAIE_INTERRUPT_DISABLE,         //no TAIE interrupt
            TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE,    //no CCR0 interrupt
            TIMER_A_DO_CLEAR);                      //clear TAR

    //Initialize CCR1 = 50% duty cycle
    TIMER_A_initCompare(TIMER_A0_BASE,                  //TA0
            TIMER_A_CAPTURECOMPARE_REGISTER_1,          //CCR1
            TIMER_A_CAPTURECOMPARE_INTERRUPT_DISABLE,   //no CCR1 interrupt
            TIMER_A_OUTPUTMODE_RESET_SET,               //Reset/Set output
            45);                                        //45/90 = 50% duty cycle

    //Initialize the ADC12B Module
    ADC12_B_init(ADC12_B_BASE,              //ADC12B
            ADC12_B_SAMPLEHOLDSOURCE_1,     //Use TA0CCR1 output as trigger (see datasheet)
            ADC12_B_CLOCKSOURCE_ADC12OSC,   //Use ADC12OSC (MODOSC) as clock
            ADC12_B_CLOCKDIVIDER_1,         //clock divide by 1
            ADC12_B_CLOCKPREDIVIDER__1,     //clock predivide by 1
            0);                             //doesn't need to be mapped

    //Use ADC in low power mode due to slow sampling
    ADC12_B_setAdcPowerMode(ADC12_B_BASE,   //ADC12B
            ADC12_B_LOWPOWERMODE);          //Use ADC12B Low Power Mode

    //Enable the ADC12B module
    ADC12_B_enable(ADC12_B_BASE);

    //Set up the sample timing for ADC12B
    ADC12_B_setupSamplingTimer(ADC12_B_BASE,    //ADC12B
            ADC12_B_CYCLEHOLD_16_CYCLES,        //MemBuf 0-7 sample/hold 16 cycles
            ADC12_B_CYCLEHOLD_4_CYCLES,         //MemBuf 8-15 sample/hold 4 cycles (default)
            ADC12_B_MULTIPLESAMPLESDISABLE);    //Disable Multiple Sampling

    //Configure Memory Buffer 0
    ADC12_B_memoryConfigure(ADC12_B_BASE,       //ADC12B
            ADC12_B_MEMORY_0,                   //Memory Buffer 0
            ADC12_B_INPUT_A2,                   //Map A2 to Memory Buffer 0
            ADC12_B_VREFPOS_AVCC_VREFNEG_VSS,   //Vref+ = Avcc, Vref- = Avss
            ADC12_B_NOTENDOFSEQUENCE,           //Is not the end of a sequence
            ADC12_B_WINDOW_COMPARATOR_ENABLE,   //Enable Window Comparator
            ADC12_B_DIFFERENTIAL_MODE_DISABLE); //Disable Differential mode

    //Set up Window Comparator Thresholds
    ADC12_B_setWindowCompAdvanced(ADC12_B_BASE, //ADC12B
            0x7FF,                              //Hi threshold = 0x7FF
            0x7FF);                             //Lo threshold = 0x7FF

    //Clear interrupt flags
    ADC12_B_clearInterrupt(ADC12_B_BASE,    //ADC12B
            2,                              //Interrupt Flag Register 2
            ADC12_B_HIIFG + ADC12_B_LOIFG); //Clear Hi and Lo flags

    //Enable window comparator interrupts
    ADC12_B_enableInterrupt(ADC12_B_BASE,   //ADC12B
            0,                              //No interrupts in ADC12IER0
            0,                              //No interrupts in ADC12IER1
            ADC12_B_HIIE + ADC12_B_LOIE);   //Enable Hi and Lo interrupts

    //start the timer
    TIMER_A_startCounter(TIMER_A0_BASE,     //Timer A0
            TIMER_A_UP_MODE);               //start the timer in up mode

    //Enable/Start sampling and conversion
    ADC12_B_startConversion(ADC12_B_BASE,       //ADC12B
            ADC12_B_START_AT_ADC12MEM0,         //Memory Buffer 0
            ADC12_B_REPEATED_SINGLECHANNEL);    //Repeated Single Channel

    while (1)
    {
        __bis_SR_register(LPM0_bits + GIE); // LPM0, ADC12_B_ISR will force exit
        __no_operation();                   // For debugger

        //if Hi interrupt fired, we only need to know when we go Lo again
        //so disable Hi interrupt, and enable Lo interrupt
        //set LED on
        if (data == 1)
        {
            //disable Hi interrupt
            ADC12_B_disableInterrupt(ADC12_B_BASE,  //ADC12B
                    0,                              //No change in ADC12IER0
                    0,                              //No change in ADC12IER1
                    ADC12_B_HIIE);                  //Disable Hi interrupt
            //enable Lo interrupt
            ADC12_B_enableInterrupt(ADC12_B_BASE,   //ADC12B
                    0,                              //No interrupts in ADC12IER0
                    0,                              //No interrupts in ADC12IER1
                    ADC12_B_LOIE);                  //Enable Lo interrupt

            //Set P1.0 LED on
            GPIO_setOutputHighOnPin(GPIO_PORT_P1,
                    GPIO_PIN0);
        }
        //if Lo interrupt fired, we only need to know when we go Hi again
        //so disable Lo interrupt, and enable Hi interrupt
        //set LED off
        else
        {
            //disable Lo interrupt
            ADC12_B_disableInterrupt(ADC12_B_BASE,  //ADC12B
                    0,                              //No change in ADC12IER0
                    0,                              //No change in ADC12IER1
                    ADC12_B_LOIE);                  //Disable Lo interrupt
            //enable Hi interrupt
            ADC12_B_enableInterrupt(ADC12_B_BASE,   //ADC12B
                    0,                              //No interrupts in ADC12IER0
                    0,                              //No interrupts in ADC12IER1
                    ADC12_B_HIIE);                  //Enable Hi interrupt

            //Set P1.0 LED off
            GPIO_setOutputLowOnPin(GPIO_PORT_P1,
                    GPIO_PIN0);
        }
    }
}

#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
    switch (__even_in_range(ADC12IV, ADC12IV_ADC12RDYIFG))
    {
        case ADC12IV_NONE:        break;    // Vector  0:  No interrupt
        case ADC12IV_ADC12OVIFG:  break;    // Vector  2:  ADC12BMEMx Overflow
        case ADC12IV_ADC12TOVIFG: break;    // Vector  4:  Conversion time overflow
        case ADC12IV_ADC12HIIFG:            // Vector  6:  ADC12BHI

            data = 1;                       //note that data was high

            __bic_SR_register_on_exit(LPM0_bits);   //Exit active CPU
            break;

        case ADC12IV_ADC12LOIFG:            // Vector  8:  ADC12BLO

            data = 0;                       //note that data was low

            __bic_SR_register_on_exit(LPM0_bits);   //Exit active CPU
            break;

        case ADC12IV_ADC12INIFG:  break;    // Vector 10:  ADC12BIN
        case ADC12IV_ADC12IFG0:             // Vector 12:  ADC12BMEM0
        case ADC12IV_ADC12IFG1:   break;    // Vector 14:  ADC12BMEM1
        case ADC12IV_ADC12IFG2:   break;    // Vector 16:  ADC12BMEM2
        case ADC12IV_ADC12IFG3:   break;    // Vector 18:  ADC12BMEM3
        case ADC12IV_ADC12IFG4:   break;    // Vector 20:  ADC12BMEM4
        case ADC12IV_ADC12IFG5:   break;    // Vector 22:  ADC12BMEM5
        case ADC12IV_ADC12IFG6:   break;    // Vector 24:  ADC12BMEM6
        case ADC12IV_ADC12IFG7:   break;    // Vector 26:  ADC12BMEM7
        case ADC12IV_ADC12IFG8:   break;    // Vector 28:  ADC12BMEM8
        case ADC12IV_ADC12IFG9:   break;    // Vector 30:  ADC12BMEM9
        case ADC12IV_ADC12IFG10:  break;    // Vector 32:  ADC12BMEM10
        case ADC12IV_ADC12IFG11:  break;    // Vector 34:  ADC12BMEM11
        case ADC12IV_ADC12IFG12:  break;    // Vector 36:  ADC12BMEM12
        case ADC12IV_ADC12IFG13:  break;    // Vector 38:  ADC12BMEM13
        case ADC12IV_ADC12IFG14:  break;    // Vector 40:  ADC12BMEM14
        case ADC12IV_ADC12IFG15:  break;    // Vector 42:  ADC12BMEM15
        case ADC12IV_ADC12IFG16:  break;    // Vector 44:  ADC12BMEM16
        case ADC12IV_ADC12IFG17:  break;    // Vector 46:  ADC12BMEM17
        case ADC12IV_ADC12IFG18:  break;    // Vector 48:  ADC12BMEM18
        case ADC12IV_ADC12IFG19:  break;    // Vector 50:  ADC12BMEM19
        case ADC12IV_ADC12IFG20:  break;    // Vector 52:  ADC12BMEM20
        case ADC12IV_ADC12IFG21:  break;    // Vector 54:  ADC12BMEM21
        case ADC12IV_ADC12IFG22:  break;    // Vector 56:  ADC12BMEM22
        case ADC12IV_ADC12IFG23:  break;    // Vector 58:  ADC12BMEM23
        case ADC12IV_ADC12IFG24:  break;    // Vector 60:  ADC12BMEM24
        case ADC12IV_ADC12IFG25:  break;    // Vector 62:  ADC12BMEM25
        case ADC12IV_ADC12IFG26:  break;    // Vector 64:  ADC12BMEM26
        case ADC12IV_ADC12IFG27:  break;    // Vector 66:  ADC12BMEM27
        case ADC12IV_ADC12IFG28:  break;    // Vector 68:  ADC12BMEM28
        case ADC12IV_ADC12IFG29:  break;    // Vector 70:  ADC12BMEM29
        case ADC12IV_ADC12IFG30:  break;    // Vector 72:  ADC12BMEM30
        case ADC12IV_ADC12IFG31:  break;    // Vector 74:  ADC12BMEM31
        case ADC12IV_ADC12RDYIFG: break;    // Vector 76:  ADC12BRDY
        default: break;
    }
}

void boardInit(void)
{
    //low power config - no floating inputs

    GPIO_setAsOutputPin(GPIO_PORT_P1, 0xFF);    //set all P1 pins as outputs
    GPIO_setOutputLowOnPin(GPIO_PORT_P1, 0xFF); //set all P1 pins low

    GPIO_setAsOutputPin(GPIO_PORT_P2, 0xFF);    //set all P2 pins as outputs
    GPIO_setOutputLowOnPin(GPIO_PORT_P2, 0xFF); //set all P2 pins low

    GPIO_setAsOutputPin(GPIO_PORT_P3, 0xFF);    //set all P3 pins as outputs
    GPIO_setOutputLowOnPin(GPIO_PORT_P3, 0xFF); //set all P3 pins low

    GPIO_setAsOutputPin(GPIO_PORT_P4, 0xFF);    //set all P4 pins as outputs
    GPIO_setOutputLowOnPin(GPIO_PORT_P4, 0xFF); //set all P4 pins low

    GPIO_setAsOutputPin(GPIO_PORT_PJ, 0xFF);    //set all PJ pins as outputs
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, 0xFF); //set all PJ pins low

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PMM_unlockLPM5(PMM_BASE);

    //DCO = 1MHz (RSEL = 0, FSEL = 0)
    CS_setDCOFreq(CS_BASE,          //CS
            CS_DCORSEL_0,           //Range select - Low Freq (1-8MHz)
            CS_DCOFSEL_0);          //Freq select - 1MHz

    //MCLK = DCO
    CS_clockSignalInit(CS_BASE,     //CS
            CS_MCLK,                //MCLK =
            CS_DCOCLK_SELECT,       //DCO (1MHz)
            CS_CLOCK_DIVIDER_1);    //divide by 1

    //SMCLK = DCO
    CS_clockSignalInit(CS_BASE,     //CS
            CS_SMCLK,               //SMCLK =
            CS_DCOCLK_SELECT,       //DCO (1MHz)
            CS_CLOCK_DIVIDER_1);    //divide by 1

    //ACLK = VLO
    CS_clockSignalInit(CS_BASE,     //CS
            CS_ACLK,                //ACLK =
            CS_VLOCLK_SELECT,       //VLO (9kHz)
            CS_CLOCK_DIVIDER_1);    //divide by 1

    //LFXT + HFXT off
    CS_LFXTOff(CS_BASE);
    CS_HFXTOff(CS_BASE);

    //clear fault flags
    CS_clearAllOscFlagsWithTimeout(CS_BASE, 5);
}
