/* --COPYRIGHT--,BSD
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************
//  MSP430FR59x Demo - MPU Write protection violation - Interrupt notification
//
//  Description: The MPU segment boundaries are defined by:
//  Border 1 = 0x6000 [MPUSEGB1 = 0x0600]
//  Border 2 = 0x8000 [MPUSEGB2 = 0x0800]
//  Segment 1 = 0x4400 - 0x5FFF
//  Segment 2 = 0x6000 - 0x7FFF
//  Segment 3 = 0x8000 - 0x13FFF
//  Segment 2 is write protected. Any write to an address in the segment 2 range
//  causes a PUC. The LED toggles after accessing SYS NMI ISR.
//
//  ACLK = n/a, MCLK = SMCLK = default DCO
//
//
//           MSP430FR5969
//         ---------------
//     /|\|               |
//      | |               |
//      --|RST            |
//        |               |
//        |           P1.0|-->LED
//
//   William Goh
//   Texas Instruments Inc.
//   October 2013
//   Built with Code Composer Studio v5.5 and MSP430Ware 1.60.01.11
//******************************************************************************
#include <msp430.h>
#include "driverlib.h"

uint8_t SYSNMIflag = 0;
uint16_t *ptr = 0;
uint16_t Data = 0;

void main(void)
{
    // Stop WDT
    WDT_A_hold(WDT_A_BASE);

    // Configure P1.0 for LED
    GPIO_setAsOutputPin(GPIO_PORT_P1,
            GPIO_PIN0);

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PMM_unlockLPM5(PMM_BASE);

    // Configure MPU Segments
    MPU_createThreeSegments(MPU_BASE,         //MPU
            0x0600,                           //boundary between 1 & 2 @ 0x6000
            0x0800,                           //boundary between 2 & 3 @ 0x8000
            MPU_READ | MPU_WRITE | MPU_EXEC,  //all access types segment 1
            MPU_READ,                         //read only segment 2
            MPU_READ | MPU_WRITE | MPU_EXEC); //all access types segment 3

    //Disable PUC on segment access violation for segment 2
    MPU_disablePUCOnViolation(MPU_BASE,       //MPU
            MPU_SECOND_SEG);                  //Segment 2

    //Enable NMI on segment access violation
    MPU_enableNMIevent(MPU_BASE);
    //Start MPU protection
    MPU_start(MPU_BASE);

    //Data value we'll use to cause a violation
    Data = 0x88;

    // Cause an MPU violation by writing to segment 2+
    ptr = (uint16_t *) 0x6002;
    *ptr = Data;

    __delay_cycles(100);

    //check for NMI flag
    while (SYSNMIflag)
    {
        //Toggle the LED on P1.0
        GPIO_toggleOutputOnPin(GPIO_PORT_P1,
                GPIO_PIN0);
        //delay so toggling is visible
        __delay_cycles(100000);
    }

    // No violation - trap here
    while (1);
}

// Sys NMI vector
#pragma vector = SYSNMI_VECTOR
__interrupt void SYSNMI_ISR(void)
{
    switch (__even_in_range(SYSSNIV, SYSSNIV_CBDIFG))
    {
        case SYSSNIV_NONE:       break;
        case SYSSNIV_SVS:        break;
        case SYSSNIV_UBDIFG:     break;
        case SYSSNIV_ACCTEIFG:   break;
        case SYSSNIV_MPUSEGPIFG: break;
        case SYSSNIV_MPUSEGIIFG: break;
        case SYSSNIV_MPUSEG1IFG: break;     //MPUSEG1IFG
        case SYSSNIV_MPUSEG2IFG:            //MPUSEG2IFG

            // Clear violation interrupt flag
            MPU_clearInterruptFlag(MPU_BASE,      //MPU
                    MPU_SEG_2_ACCESS_VIOLATION);  //Clear Seg2 violation flag

            SYSNMIflag = 1;                 // Set flag
            break;

        case SYSSNIV_MPUSEG3IFG: break;     // MPUSEG3IFG
        case SYSSNIV_VMAIFG:     break;
        case SYSSNIV_JMBINIFG:   break;
        case SYSSNIV_JMBOUTIFG:  break;
        case SYSSNIV_CBDIFG:     break;
        default: break;
    }
}
